==> neo4j-front-end-5.13.0-sources.jar: org/neo4j/cypher/internal/frontend/phases/CopyQuantifiedPathPatternPredicatesToJuxtaposedNodes.scala <==
/*
 * Copyright (c) "Neo4j"
 * Neo4j Sweden AB [https://neo4j.com]
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.neo4j.cypher.internal.frontend.phases

import org.neo4j.cypher.internal.ast.semantics.SemanticFeature
import org.neo4j.cypher.internal.frontend.phases.factories.PlanPipelineTransformerFactory
import org.neo4j.cypher.internal.rewriting.conditions.AndRewrittenToAnds
import org.neo4j.cypher.internal.rewriting.conditions.noUnnamedNodesAndRelationships
import org.neo4j.cypher.internal.rewriting.rewriters.NoNodeOrRelationshipPredicates
import org.neo4j.cypher.internal.rewriting.rewriters.QppsHavePaddedNodes
import org.neo4j.cypher.internal.rewriting.rewriters.computeDependenciesForExpressions.ExpressionsHaveComputedDependencies
import org.neo4j.cypher.internal.util.Rewriter
import org.neo4j.cypher.internal.util.StepSequencer

case object CopyQuantifiedPathPatternPredicatesToJuxtaposedNodes
    extends StatementRewriter with StepSequencer.Step with PlanPipelineTransformerFactory {

  case object QppPredicatesCopiedToJuxtaposedNodes extends StepSequencer.Condition

  override def preConditions: Set[StepSequencer.Condition] = Set(
    ExpressionsHaveComputedDependencies,
    StatementCondition.wrap(noUnnamedNodesAndRelationships),
    QppsHavePaddedNodes,
    NoNodeOrRelationshipPredicates,
    AndRewrittenToAnds,
    AmbiguousNamesDisambiguated
  )

  override def postConditions: Set[StepSequencer.Condition] =
    Set(QppPredicatesCopiedToJuxtaposedNodes)

  override def invalidatedConditions: Set[StepSequencer.Condition] =
    Set(ExpressionsHaveComputedDependencies)

  def instance(from: BaseState, context: BaseContext): Rewriter =
    CopyQuantifiedPathPatternPredicatesToJuxtaposedNodesRewriter.instance

  override def getTransformer(
    pushdownPropertyReads: Boolean,
    semanticFeatures: Seq[SemanticFeature]
  ): Transformer[BaseContext, BaseState, BaseState] = this
}

==> neo4j-front-end-5.26.12-sources.jar: org/neo4j/cypher/internal/frontend/phases/CopyQuantifiedPathPatternPredicatesToJuxtaposedNodes.scala <==
/*
 * Copyright (c) "Neo4j"
 * Neo4j Sweden AB [https://neo4j.com]
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.neo4j.cypher.internal.frontend.phases

import org.neo4j.cypher.internal.ast.semantics.SemanticFeature
import org.neo4j.cypher.internal.frontend.phases.factories.PlanPipelineTransformerFactory
import org.neo4j.cypher.internal.rewriting.conditions.AndRewrittenToAnds
import org.neo4j.cypher.internal.rewriting.conditions.noUnnamedNodesAndRelationships
import org.neo4j.cypher.internal.rewriting.rewriters.QuantifiedPathPatternNodeInsertRewriter
import org.neo4j.cypher.internal.rewriting.rewriters.computeDependenciesForExpressions.ExpressionsHaveComputedDependencies
import org.neo4j.cypher.internal.rewriting.rewriters.normalizePredicates
import org.neo4j.cypher.internal.util.Rewriter
import org.neo4j.cypher.internal.util.StepSequencer
import org.neo4j.cypher.internal.util.StepSequencer.DefaultPostCondition

case object CopyQuantifiedPathPatternPredicatesToJuxtaposedNodes
    extends StatementRewriter with StepSequencer.Step with DefaultPostCondition with PlanPipelineTransformerFactory {

  override def preConditions: Set[StepSequencer.Condition] = Set(
    ExpressionsHaveComputedDependencies,
    StatementCondition.wrap(noUnnamedNodesAndRelationships),
    QuantifiedPathPatternNodeInsertRewriter.completed,
    normalizePredicates.completed,
    AndRewrittenToAnds,
    Namespacer.completed
  )

  override def invalidatedConditions: Set[StepSequencer.Condition] =
    Set(ExpressionsHaveComputedDependencies)

  def instance(from: BaseState, context: BaseContext): Rewriter =
    CopyQuantifiedPathPatternPredicatesToJuxtaposedNodesRewriter.instance

  override def getTransformer(
    pushdownPropertyReads: Boolean,
    semanticFeatures: Seq[SemanticFeature]
  ): Transformer[BaseContext, BaseState, BaseState] = this
}
